#include <stdio.h>
#include <stdint.h>

#include "hid.h"
#include "spi.h"
#include "nrf24l01.h"

#define	_CH             1   // Channel 0..125
#define	_Address_Width  5   // 3..5
#define _Buffer_Size    8   // 1..32

void NRF24L01_Receive(char Buf[_Buffer_Size]) {
	uint8_t status;
	
	//printf("NRF24L01_Receive()\r\n");
	
	NRF24L01_CE_HIGH;
	Delay_us(130);

	do {
		status = NRF24L01_Get_Status();
	} while ((status & _RX_DR) != _RX_DR);

	NRF24L01_CE_LOW;

	NRF24L01_Read_RX_Buf(Buf, _Buffer_Size);
	NRF24L01_Clear_Interrupts();
}


void NRF24L01_Send(char Buf[_Buffer_Size]) {
	//printf("NRF24L01_Send()\r\n");
	
	NRF24L01_Write_TX_Buf(Buf, _Buffer_Size);

	NRF24L01_RF_TX();

	while ((NRF24L01_Get_Status() & _TX_DS) != _TX_DS);

	NRF24L01_Clear_Interrupts();
}

hid_context device;

int main(int argc, char **argv)
{
	int ret;
	int n = 0;
	char Buf[_Buffer_Size];
	//char Address[_Address_Width] = { 0x11, 0x22, 0x33, 0x44, 0x55 };
	char Address[_Address_Width] = { 0xAB, 0xDE, 0x54, 0xEF, 0x13 };
	
	printf("Open STD device... ");
	ret = hid_find(&device, 0x1FC9, 0x0007, 0xFF00, 0);
	
	if (ret == 0)
	{
		//  :
		printf("\r\n");
		printf("ERROR: Device not found!\r\n");
		return 0;
	}
	else
	{
		printf("OK!\r\n");
	}

	NRF24L01_Init(_RX_MODE, _CH, _1Mbps, Address, _Address_Width, _Buffer_Size);

	while (1) {
		NRF24L01_Receive(Buf);
		
		//printf("Received! Buffer: %02x %02x %02x %02x %02x %02x %02x %02x\r\n",
		//	Buf[0], Buf[1], Buf[2], Buf[3], 
		//	Buf[4], Buf[5], Buf[6], Buf[7]);
		printf("Received (%d): H: %.1f%% T: %.1fC\r\n",
			n, *(float*)&Buf[0], *(float*)&Buf[4]);
			
		n++;
	}

	hid_close(&device);
}
